/*$Id: d_cap.h,v 16.19 2000/02/03 08:41:50 al Exp $ -*- C++ -*-
 * capacitance devices:
 *	self-capacitance (C device)
 *	trans-capacitance (non-spice charge transfer device)
 */
#ifndef D_CAP_H
#define D_CAP_H
#include "e_storag.h"
/*--------------------------------------------------------------------------*/
class DEV_CAPACITANCE : public STORAGE {
protected:
  explicit DEV_CAPACITANCE(const DEV_CAPACITANCE& p) :STORAGE(p) {}
public:
  explicit DEV_CAPACITANCE()	:STORAGE() {}
protected: // override virtual
  char	   id_letter()const	{return 'C';}
  const char* dev_type()const	{return "capacitor";}
  int	   numnodes()const	{return 2;}
  bool	   is_1port()const	{return true;}
  CARD*	   clone()const		{return new DEV_CAPACITANCE(*this);}
  //void   parse(CS&);		//ELEMENT
  //void   print(OMSTREAM,int)const; //ELEMENT
  //void   expand();		//CARD/nothing
  //void   precalc()		//STORAGE

  void     dc_begin()		{begin_core();}
  void     tr_begin()		{begin_core();}
  void     tr_restore()		{begin_core();}
  //void   dc_advance();	//STORAGE
  //void   tr_advance();	//STORAGE
  //bool   tr_needs_eval();	//STORAGE
  //void   tr_queue_eval();	//STORAGE
  bool	   do_tr();
  void	   tr_load()		{tr_load_passive();}
  double   tr_review();
  //void   tr_accept();		//CARD/nothing
  void	   tr_unload()		{tr_unload_passive();}
  //double tr_amps()const	//ELEMENT
  double   tr_involts()const	{return tr_outvolts();}
  double   tr_involts_limited()const {return tr_outvolts_limited();}

  void	   ac_begin()		{_ev = _y0.f1;}
  void	   do_ac();
  void	   ac_load()		{ac_load_passive();}
  COMPLEX  ac_involts()const	{return ac_outvolts();}
protected:
  void	   integrate();
};
/*--------------------------------------------------------------------------*/
class DEV_TRANSCAP : public DEV_CAPACITANCE {
private:
  explicit DEV_TRANSCAP(const DEV_TRANSCAP& p) :DEV_CAPACITANCE(p){untested();}
public:
  explicit DEV_TRANSCAP()	:DEV_CAPACITANCE() {}
private: // override virtual
  char     id_letter()const	{return '\0';}
  const char* dev_type()const	{return "tcap";}
  int	   numnodes()const	{return 4;}
  bool	   is_1port()const	{untested(); return false;}
  bool	   is_2port()const	{return true;}
  bool	   f_is_value()const	{untested(); return true;}
  CARD*	   clone()const		{untested(); return new DEV_TRANSCAP(*this);}
  //void   parse(CS&);		//ELEMENT
  //void   print(OMSTREAM,int)const; //ELEMENT
  //void   expand();		//CARD/nothing
  //void   precalc()		//STORAGE

  //void   dc_begin();		//DEV_CAPACITANCE
  //void   tr_begin();		//DEV_CAPACITANCE
  //void   tr_restore();	//DEV_CAPACITANCE
  //void   dc_advance();	//STORAGE
  //void   tr_advance();	//STORAGE
  //bool   tr_needs_eval();	//STORAGE
  //void   tr_queue_eval();	//STORAGE
  //bool   do_tr();		//DEV_CAPACITANCE
  void	   tr_load()		{tr_load_active();}
  //double tr_review();		//DEV_CAPACITANCE
  //void   tr_accept();		//CARD/nothing
  //void   tr_unload();		//DEV_CAPACITANCE
  //double tr_amps()const	//ELEMENT
  double   tr_involts()const	{return dn_diff(n[IN1].v0(),n[IN2].v0());}
  double   tr_involts_limited()const
			{untested(); return volts_limited(n[IN1],n[IN2]);}

  //void   ac_begin();		//DEV_CAPACITANCE
  //void   do_ac();		//DEV_CAPACITANCE
  void	   ac_load()		{untested(); ac_load_active();}
  //COMPLEX ac_involts()const;	//DEV_CAPACITANCE
};
/*--------------------------------------------------------------------------*/
// BUG:: doesn't model dynamic effects of control.
class DEV_VCCAP : public DEV_CAPACITANCE {
private:
  explicit DEV_VCCAP(const DEV_VCCAP& p) :DEV_CAPACITANCE(p) {untested();}
public:
  explicit DEV_VCCAP()		:DEV_CAPACITANCE() {}
private: // override virtual
  char     id_letter()const	{return '\0';}
  const char* dev_type()const	{return "vccap";}
  int	   numnodes()const	{return 4;}
  bool	   is_2port()const	{return true;}
  bool	   f_is_value()const	{untested(); return true;}
  CARD*	   clone()const		{return new DEV_VCCAP(*this);}
  //void   parse(CS&);		//ELEMENT
  //void   print(OMSTREAM,int)const; //ELEMENT
  //void   expand();		//CARD/nothing
  //void   precalc()		//STORAGE

  //void   dc_begin();		//DEV_CAPACITANCE
  //void   tr_begin();		//DEV_CAPACITANCE
  //void   tr_restore();	//DEV_CAPACITANCE
  //void   dc_advance();	//STORAGE
  //void   tr_advance();	//STORAGE
  //bool   tr_needs_eval();	//STORAGE
  //void   tr_queue_eval();	//STORAGE
  bool     do_tr();
  //void   tr_load();		//DEV_CAPACITANCE
  //double tr_review();		//DEV_CAPACITANCE
  //void   tr_accept();		//CARD/nothing
  //void   tr_unload();		//DEV_CAPACITANCE
  //double tr_amps()const	//ELEMENT
  double   tr_involts()const	{return dn_diff(n[IN1].v0(),n[IN2].v0());}
  double tr_involts_limited()const{unreachable(); return volts_limited(n[IN1],n[IN2]);}

  //void   ac_begin();		//DEV_CAPACITANCE
  //void   do_ac();		//DEV_CAPACITANCE
  //void   ac_load();		//DEV_CAPACITANCE
  //COMPLEX ac_involts()const;	//DEV_CAPACITANCE
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
