/*$Id: d_mos4.cc,v 16.14 1999/12/19 22:52:34 al Exp $ -*- C++ -*-
 * Spice BSIM1 model
 * derived from Spice3f4,Copyright 1990 Regents of the University of California
 * 1985 Hong J. Park, Thomas L. Quarles
 */
#include "d_mos4.h"
/*--------------------------------------------------------------------------*/
// this model doesn't do temperature (Spice compatibility)
const double CONSTvt0 = K*300.15/Q;
/*--------------------------------------------------------------------------*/
SDP_MOS4::SDP_MOS4(const MOS_COMMON* c)
  :SDP_MOS_BASE(c)
{
  const MODEL_MOS4* m = prechecked_cast<const MODEL_MOS4*>(c->model());
  assert(m);

  le -= m->dl;
  we -= m->dw;
  cgate = m->cox * we * le;
  double L = le/MICRON2METER;
  double W = we/MICRON2METER;
  double CoxWoverL = 1e-4 * m->cox * we / le;

  phi = m->phi(L, W);
  vfb = m->vfb(L, W);
  k1 = m->k1(L, W);
  k2 = m->k2(L, W);
  eta = m->eta(L, W);
  etaB = m->etaB(L, W);
  etaD = m->etaD(L, W);
  betaZero = m->mobZero * CoxWoverL;
  betaZeroB = m->mobZeroB(L, W) * CoxWoverL;
  betaVdd = m->mobVdd(L, W) * CoxWoverL;
  betaVddB = m->mobVddB(L, W) * CoxWoverL;
  betaVddD = m->mobVddD(L, W) * CoxWoverL;
  ugs = m->ugs(L, W);
  ugsB = m->ugsB(L, W);
  uds = m->uds(L, W);
  udsB = m->udsB(L, W);
  udsD = m->udsD(L, W);
  n0 = m->n0(L, W);
  nB = m->nB(L, W);
  nD = m->nD(L, W);

  phi = std::max(phi, .1);
  k1  = std::max(k1, 0.);
  k2  = std::max(k2, 0.);

  vt0 = vfb + phi + k1 * sqrt(phi) - k2 * phi;
}
/*--------------------------------------------------------------------------*/
MODEL_MOS4::MODEL_MOS4()
  :MODEL_MOS_BASE(),
   mobZero(0.),
   dl(0.),	// binned parameters have initializing default constructors
   dw(0.),
   tox(0.),
   vdd(0.),
   wdf(0.),
   dell(0.),
   temp(300.15),
   xpart(0.),
   cox(0.)
{
}
/*--------------------------------------------------------------------------*/
void MODEL_MOS4::parse_params(CS& cmd)
{
  parse_params_base(cmd);
  vfb.get(cmd, "VFB");
  phi.get(cmd, "PHI");
  k1.get(cmd, "K1");
  k2.get(cmd, "K2");
  eta.get(cmd, "ETA");
  etaB.get(cmd, "X2E");
  etaD.get(cmd, "X3E");
  cmd.get("MUZ",   &mobZero);
  mobZeroB.get(cmd, "X2MZ");
  mobVdd.get(cmd, "MUS");
  mobVddB.get(cmd, "X2MS");
  mobVddD.get(cmd, "X3MS");
  ugs.get(cmd, "U0");
  ugsB.get(cmd, "X2U0");
  uds.get(cmd, "U1");
  udsB.get(cmd, "X2U1");
  udsD.get(cmd, "X3U1");
  n0.get(cmd, "N0");
  nB.get(cmd, "NB");
  nD.get(cmd, "ND");
  cmd.get("DL",   &dl,	mSCALE, MICRON2METER);
  cmd.get("DW",   &dw,	mSCALE, MICRON2METER);
  cmd.get("TOX",  &tox,	mSCALE, MICRON2METER);
  cmd.get("VDD",  &vdd);
  cmd.get("WDF",  &wdf);
  cmd.get("DELL", &dell);
  cmd.get("TEMP", &temp);
  cmd.get("XPART",&xpart);
}
/*--------------------------------------------------------------------------*/
void MODEL_MOS4::post_parse()
{
  post_parse_base();
  cox = 3.453e-11 /*E_OX*/ / tox; // incorrect # for Spice compatibility
  pb = std::min(pb, .1);
  pbsw = std::min(pbsw, .1);
}
/*--------------------------------------------------------------------------*/
void MODEL_MOS4::print(OMSTREAM& o, int)const
{
  print_base_begin(o, LEVEL);
  vfb.print(o, "vfb");
  phi.print(o, "phi");
  k1.print(o, "k1");
  k2.print(o, "k2");
  eta.print(o, "eta");
  etaB.print(o, "x2e");
  etaD.print(o, "x3e");
  o << "  muz=" << mobZero;
  mobZeroB.print(o, "x2mz");
  mobVdd.print(o, "mus");
  mobVddB.print(o, "x2ms");
  mobVddD.print(o, "x3ms");
  ugs.print(o, "u0");
  ugsB.print(o, "x2u0");
  uds.print(o, "u1");
  udsB.print(o, "x2u1");
  udsD.print(o, "x3u1");
  n0.print(o, "n0");
  nB.print(o, "nb");
  nD.print(o, "nd");
  o << "  dl="   << dl/MICRON2METER;
  o << "  dw="   << dw/MICRON2METER;
  o << "  tox="	 << tox/MICRON2METER;
  o << "  vdd="	 << vdd;
  o << "  wdf="  << wdf;
  o << "  dell=" << dell;
  o << "  temp=" << temp;
  o << "  xpart="<< xpart;
  print_base_mid(o);
  print_base_end(o);
}
/*--------------------------------------------------------------------------*/
void MODEL_MOS4::tr_eval(COMPONENT* brh)const
{
  DEV_MOS* d = prechecked_cast<DEV_MOS*>(brh);
  assert(d);
  const MOS_COMMON* c = prechecked_cast<const MOS_COMMON*>(d->common());
  assert(c);
  const SDP_MOS4* b = prechecked_cast<const SDP_MOS4*>(c->sdp);
  assert(b);
  const MODEL_MOS4* m = this;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */ 
  trace3("", d->vds, d->vgs, d->vbs);
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  trace4("", c->lo, m->dl, c->wo, m->dw);
  trace3("", b->ugs, b->ugsB, d->vbs);
  double Ugs = b->ugs + b->ugsB * d->vbs;
  double dUgsdVbs;
  {if(Ugs <= 0){
    untested();
    Ugs = 0;
    dUgsdVbs = 0.0;
  }else{
    dUgsdVbs = b->ugsB;
  }}
  trace2("", Ugs, dUgsdVbs);

  double Uds = b->uds + b->udsB * d->vbs + b->udsD * (d->vds - m->vdd);
  double dUdsdVbs;
  double dUdsdVds;
  {if(Uds <= 0){    
    untested();
    Uds = 0.0;
    dUdsdVbs = dUdsdVds = 0.0;
  }else{
    double Leff = b->le * 1e6; /* Leff in um */
    Uds  =  Uds / Leff;
    dUdsdVbs = b->udsB / Leff;
    dUdsdVds = b->udsD / Leff;
  }}
  trace3("", Uds, dUdsdVbs, dUdsdVds);
  
  double Vpb;
  {if(d->vbs <= 0){
    Vpb = b->phi - d->vbs;
    d->sbfwd = false;
  }else{
    Vpb = b->phi;
    d->sbfwd = true;
  }}
  double SqrtVpb = sqrt(Vpb);
  trace2("", Vpb, SqrtVpb);
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* threshold voltage */
  double dVthdVbs;
  double dVthdVds;
  {
    double Eta = b->eta + b->etaB * d->vbs + b->etaD * (d->vds - m->vdd);
    double dEtadVds;
    double dEtadVbs;
    {if(Eta <= 0){   
      Eta  = 0; 
      dEtadVds = dEtadVbs = 0.0 ;
    }else if (Eta > 1){
      untested();
      Eta = 1;
      dEtadVds = dEtadVbs = 0;
    }else{ 
      untested();
      dEtadVds = b->etaD;
      dEtadVbs = b->etaB;
    }}
    trace3("", Eta, dEtadVds, dEtadVbs);
    d->von = b->vfb + b->phi + b->k1 * SqrtVpb - b->k2 * Vpb - Eta * d->vds;
    dVthdVds = -Eta - dEtadVds * d->vds;
    dVthdVbs = b->k2 - 0.5 * b->k1 / SqrtVpb - dEtadVbs * d->vds;
    d->vgst  = d->vgs - d->von;
    trace4("", d->von, dVthdVds, dVthdVbs, d->vgst);
  }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  double G = 1. - 1./(1.744+0.8364 * Vpb);
  double A = 1. + 0.5*G*b->k1/SqrtVpb;
  A = std::max(A, 1.0);   /* Modified */
  double Arg = std::max((1 + Ugs * d->vgst), 1.0);
  double dGdVbs = -0.8364 * (1-G) * (1-G);
  double dAdVbs = 0.25 * b->k1 / SqrtVpb *(2*dGdVbs + G/Vpb);
  trace3("", G, A, Arg);
  trace2("", dGdVbs, dAdVbs);
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* ids and derivatives calculation */
  {if (d->vgst < 0){
    d->cutoff = true;
    /* cutoff */
    d->ids = 0;
    d->gm = 0;
    d->gds = 0;
    d->gmb = 0;
    d->vdsat = 0;
    trace4("cutoff", d->ids, d->gm, d->gds, d->gmb);
  }else{
    d->cutoff = false;
    /* Quadratic Interpolation for Beta0 (Beta at d->vgs  =  0, vds=Vds) */
    double Beta0;
    double dBeta0dVds;
    double dBeta0dVbs;
    {
      trace2("", m->tox, m->cox*1e-4);
      trace3("", b->betaVdd, b->betaVddB, d->vbs);
      double BetaVdd = (b->betaVdd + b->betaVddB * d->vbs);
      double dBetaVdd_dVds = std::max(b->betaVddD, 0.0); /* Modified */
      trace2("", BetaVdd, dBetaVdd_dVds);
      {if(d->vds > m->vdd){
	Beta0      = BetaVdd + dBetaVdd_dVds * (d->vds - m->vdd);
	dBeta0dVds = dBetaVdd_dVds;
	dBeta0dVbs = b->betaVddB;
	trace3("vds>vdd", Beta0, dBeta0dVds, dBeta0dVbs);
      }else{
	double Beta_Vds_0 = (b->betaZero + b->betaZeroB * d->vbs);
	double VddSquare = m->vdd * m->vdd;
	double C1 = (-BetaVdd + Beta_Vds_0 + dBetaVdd_dVds*m->vdd) / VddSquare;
	double C2 = 2 * (BetaVdd - Beta_Vds_0) / m->vdd - dBetaVdd_dVds;
	trace4("", Beta_Vds_0, VddSquare, C1, C2);
	double dBeta_Vds_0_dVbs = b->betaZeroB;
	double dBetaVdd_dVbs = b->betaVddB;
	double dC1dVbs = (dBeta_Vds_0_dVbs - dBetaVdd_dVbs) / VddSquare;
	double dC2dVbs = dC1dVbs * (-2) * m->vdd;
	trace4("", dBeta_Vds_0_dVbs, dBetaVdd_dVbs, dC1dVbs, dC2dVbs);
	Beta0	   = (C1 * d->vds + C2) * d->vds + Beta_Vds_0;
	dBeta0dVds = 2*C1*d->vds + C2;
	dBeta0dVbs = dC1dVbs * d->vds * d->vds 
	  + dC2dVbs * d->vds + dBeta_Vds_0_dVbs;
	trace3("vds<vdd", Beta0, dBeta0dVds, dBeta0dVbs);
      }}
    }

    /*Beta  =  Beta0 / ( 1 + Ugs * d->vgst );*/
    double Beta	     = Beta0 / Arg ;
    double dBetadVgs = -Beta * Ugs / Arg;
    double dBetadVds = dBeta0dVds / Arg - dBetadVgs * dVthdVds ;
    double dBetadVbs = dBeta0dVbs / Arg 
      + Beta * Ugs * dVthdVbs / Arg - Beta * d->vgst * dUgsdVbs / Arg;
    trace4("", Beta, dBetadVgs, dBetadVds, dBetadVbs);

    /*d->vdsat  = std::max(d->vgst / ( A + Uds * d->vgst ),  0.0);*/
    double Vc = Uds * d->vgst / A;
    if(Vc < 0.0){
      untested();
      Vc=0.0;
    }

    double Term1 = sqrt(1 + 2 * Vc);
    double K = 0.5 * (1 + Vc + Term1);
    d->vdsat = std::max(d->vgst / (A * sqrt(K)) , 0.0);
    trace4("", Vc, Term1, K, d->vdsat);

    {if(d->vds < d->vdsat){		/* Triode Region */
      d->saturated = false;
      /*Argl1  =  1 + Uds * d->vds;*/
      double Argl1 = std::max((1 + Uds * d->vds), 1.);
      double Argl2 = d->vgst - 0.5 * A * d->vds;
      trace2("", Argl1, Argl2);
      d->ids = Beta * Argl2 * d->vds / Argl1;
      d->gm   = (dBetadVgs * Argl2 * d->vds + Beta * d->vds) / Argl1;
      d->gds  = (dBetadVds * Argl2 * d->vds 
	      + Beta * (d->vgst - d->vds * dVthdVds - A * d->vds)
	      - d->ids * (d->vds * dUdsdVds + Uds)) /  Argl1;
      d->gmb = (dBetadVbs * Argl2 * d->vds 
	      + Beta * d->vds * (-dVthdVbs - 0.5 * d->vds * dAdVbs)
	      - d->ids * d->vds * dUdsdVbs) / Argl1;
      trace4("triode", d->ids, d->gm, d->gds, d->gmb);
    }else{			/* Pinchoff (Saturation) Region */
      d->saturated = true;
      double Args1   =  1. + 1. / Term1;
      double dVcdVgs =  Uds / A;
      double dVcdVds =  d->vgst * dUdsdVds / A - dVcdVgs * dVthdVds;
      double dVcdVbs =  (d->vgst * dUdsdVbs 
			 - Uds * (dVthdVbs + d->vgst * dAdVbs / A )) / A;
      double dKdVc   =  0.5 * Args1;
      double dKdVgs  =  dKdVc * dVcdVgs;
      double dKdVds  =  dKdVc * dVcdVds;
      double dKdVbs  =  dKdVc * dVcdVbs;
      double Args2   =  d->vgst / A / K;
      double Args3   =  Args2 * d->vgst;
      trace3("", Args1, Args2, Args3);
      trace3("", dVcdVgs, dVcdVds, dVcdVbs);
      trace4("", dKdVc, dKdVgs, dKdVds, dKdVbs);
      d->ids =  0.5 * Beta * Args3;
      d->gm = 0.5 * Args3 * dBetadVgs + Beta * Args2 - d->ids * dKdVgs / K;
      d->gds = 0.5*Args3*dBetadVds - Beta*Args2*dVthdVds - d->ids*dKdVds/K;
      d->gmb = 0.5 * dBetadVbs * Args3 - Beta * Args2 *dVthdVbs
	- d->ids * (dAdVbs / A + dKdVbs / K);
      trace4("sat", d->ids, d->gm, d->gds, d->gmb);
    }}
  }}
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* SubthresholdComputation */

/* The following 'if' statement has been modified so that subthreshold  *
 * current computation is always executed unless N0 >= 200. This should *
 * get rid of the Ids kink seen on Ids-Vgs plots at low Vds.            *
 *                                                Peter M. Lee          *
 *                                                6/8/90                *
 *  Old 'if' statement:  (reversed)                                     *
 *  if( (N0 >=  200) || (d->vgst < Vcut ) || (d->vgst > (-0.5*Vcut)))   */
  
  //double Vcut  =  - 40. * b->n0 * CONSTvt0 ;
  {if (b->n0 < 200){
    double N = b->n0 + b->nB*d->vbs + b->nD*d->vds; /* subthreshold slope */
    trace4("", b->n0, b->nB, b->nD, N);
    if (N < 0.5){
      untested();
      N = 0.5;
    }
    const double Vtsquare = CONSTvt0 * CONSTvt0 ;
    const double nvt0 = N * CONSTvt0;
    double Warg1 = exp(-d->vds / CONSTvt0);
    double Wds   = 1 - Warg1;
    double Wgs   = exp( d->vgst / nvt0);
    double Warg2  = 6.04965 * Vtsquare * b->betaZero;
    double Ilimit = 4.5 * Vtsquare * b->betaZero;
    double Iexp   = Warg2 * Wgs * Wds;
    d->ids += Ilimit * Iexp / (Ilimit + Iexp);
    double Temp1  = Ilimit / (Ilimit + Iexp);
    Temp1  =  Temp1 * Temp1;
    double Temp3  = Ilimit / (Ilimit + Wgs * Warg2);
    Temp3 = Temp3 * Temp3 * Warg2 * Wgs;
    /*    if ( Temp3 > Ilimit ) Temp3=Ilimit;*/
    d->gm  += Temp1 * Iexp / nvt0;
    /* gds term has been modified to prevent blow up at Vds=0 */
    d->gds += Temp3
      * (Wds / nvt0 * (dVthdVds + d->vgst * b->nD / N)
	 + Warg1 / CONSTvt0);
    d->gmb -= Temp1 * Iexp * (dVthdVbs + d->vgst * b->nB / N) / nvt0;
    trace3("", CONSTvt0, Vtsquare, nvt0);
    trace4("", Warg1, Wds, Wgs, Warg2);
    trace4("", Ilimit, Iexp, Temp1, Temp3);
    trace4("sub", d->ids, d->gm, d->gds, d->gmb);
  }else{
    untested();
  }}
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  /* Some Limiting of DC Parameters */
  if(d->ids < 0.0) d->ids = 0.0;
  if(d->gm  < 0.0) d->gm  = 0.0;
  if(d->gds < 0.0) d->gds = 0.0;
  if(d->gmb < 0.0) d->gmb = 0.0;
  trace4("final", d->ids, d->gm, d->gds, d->gmb);
  do_charge(brh);
}
/*--------------------------------------------------------------------------*/
void MODEL_MOS4::do_charge(COMPONENT* brh)const
{
  DEV_MOS* d = prechecked_cast<DEV_MOS*>(brh);
  assert(d);
  const MOS_COMMON* c = prechecked_cast<const MOS_COMMON*>(d->common());
  assert(c);
  const SDP_MOS4* b = prechecked_cast<const SDP_MOS4*>(c->sdp);
  assert(b);
  const MODEL_MOS4* m = this;
  
  double qg;
  double qb;
  double qd;

  double cggb;
  double cgdb;
  double cgsb;

  double cbgb;
  double cbdb;
  double cbsb;

  double cdgb;
  double cddb;
  double cdsb;
  
  double Vpb;
  {if(d->vbs <= 0){
    Vpb = b->phi - d->vbs;
  }else{
    Vpb = b->phi;
  }}
  double SqrtVpb = sqrt(Vpb);
  trace4("", Vpb, SqrtVpb, b->phi, d->vbs);
  double G = 1. - 1. / (1.744 + 0.8364 * Vpb);
  double A = 1. + 0.5 * G * b->k1 / SqrtVpb;
  A  = std::max(A, 1.0);   /* Modified */
  /*Arg  = 1 + Ugs * d->vgst;*/

  trace3("", G, A, b->phi);
  trace1("", m->xpart);
  
  double Vth0 = b->vfb + b->phi + b->k1 * SqrtVpb; // almost same as d->von
  double Vgs_Vth = d->vgs - Vth0; // almost same as d->vgst
  trace2("", Vth0, Vgs_Vth);
  double Arg1 = A * d->vds;
  double Arg2 = Vgs_Vth - 0.5 * Arg1;
  double Arg3 = d->vds - Arg1;
  trace3("", Arg1, Arg2, Arg3);
  double dVthdVbs = -0.5 * b->k1 / SqrtVpb;
  double dAdVbs = 0.5 * b->k1 * (0.5*G/Vpb - 0.8364*(1-G)*(1-G)) / SqrtVpb;
  trace2("", dVthdVbs, dAdVbs);
  double Ent = std::max(Arg2,1.0e-8);
  double dEntdVds = -0.5 * A;
  double dEntdVbs = -dVthdVbs - 0.5 * d->vds * dAdVbs;
  trace3("", Ent, dEntdVds, dEntdVbs);
  double VdsPinchoff = std::max(Vgs_Vth / A, 0.0);
  double Vgb  = d->vgs - d->vbs ;
  double Vgb_Vfb  = Vgb - b->vfb;
  trace3("", VdsPinchoff, Vgb, Vgb_Vfb);
  
  {if(Vgb_Vfb < 0){			/* Accumulation Region */
    untested();
    qg   = b->cgate * Vgb_Vfb;
    qb   = -qg;
    qd   = 0. ;
    cggb = b->cgate;
    cgdb = 0.;
    cgsb = 0.;
    cbgb = -b->cgate;
    cbdb = 0.;
    cbsb = 0.;
    cdgb = 0.;
    cddb = 0.;
    cdsb = 0.;
    trace4("", qg, cggb, cgdb, cgsb);
    trace4("", qb, cbgb, cbdb, cbsb);
    trace4("acc", qd, cdgb, cddb, cdsb);
  }else if (d->vgs < Vth0){		/* Subthreshold Region */
    qg   = 0.5*b->cgate*b->k1*b->k1*(-1+sqrt(1+4*Vgb_Vfb/(b->k1*b->k1)));
    cggb = b->cgate / sqrt(1 + 4 * Vgb_Vfb / (b->k1 * b->k1));
    cgdb = cgsb = 0.;
    qb   = -qg;
    cbgb = -cggb;
    cbdb = cbsb = 0.0;
    qd   = 0.;
    cdgb = cddb = cdsb = 0.0;
    trace4("", qg, cggb, cgdb, cgsb);
    trace4("", qb, cbgb, cbdb, cbsb);
    trace4("sub", qd, cdgb, cddb, cdsb);
  }else if (d->vds < VdsPinchoff){	/* triode region */
    double EntSquare = Ent * Ent;
    trace1("tri", EntSquare);
    double Argl1 = 1.2e1 * EntSquare;
    double Argl2 = 1.0 - A;
    double Argl3 = Arg1 * d->vds;
    trace3("", Argl1, Argl2, Argl3);
    double Argl5;
    {if (Ent > 1.0e-8){
      Argl5 = Arg1 / Ent;
    }else{   
      untested();
      Argl5 = 2.0;
    }}
    double Argl7 = Argl5 / 1.2e1;
    double Argl8 = 6.0 * Ent;
    trace3("", Argl5, Argl7, Argl8);

    qg = b->cgate * (d->vgs - b->vfb - b->phi - 0.5 * d->vds + d->vds * Argl7);
    cggb = b->cgate * (1.0 - Argl3 / Argl1);
    cgdb = b->cgate * (-0.5 + Arg1 / Argl8 - Argl3 * dEntdVds / Argl1);
    double cgbb = b->cgate * (d->vds*d->vds*dAdVbs*Ent - Argl3*dEntdVbs)/Argl1;
    cgsb = -(cggb + cgdb + cgbb);
    trace4("", qg, cggb, cgdb, cgsb);

    qb = b->cgate * (-Vth0 + b->vfb + b->phi + 0.5 * Arg3 - Arg3 * Argl7);
    cbgb = b->cgate * Argl3 * Argl2 / Argl1;
    cbdb = b->cgate * Argl2 * (0.5 - Arg1 / Argl8 + Argl3 * dEntdVds / Argl1);
    double cbbb = -b->cgate * (dVthdVbs + 0.5 * d->vds * dAdVbs
	+d->vds*d->vds*((1.0-2.0*A)*dAdVbs*Ent-Argl2*A*dEntdVbs)/Argl1);
    cbsb = -(cbgb + cbdb + cbbb);
    trace4("", qb, cbgb, cbdb, cbsb);

    {if (m->xpart >= 1){
      /*0/100 partitioning for drain/source chArges at saturation region*/
      double Argl9 = 0.125 * Argl5 * Argl5; //t
      qd = -b->cgate * (0.5 * Vgs_Vth - 0.75 * Arg1 + 0.125 * Arg1 * Argl5);
      cdgb = -b->cgate * (0.5 - Argl9);
      cddb = b->cgate * (0.75 * A - 0.25 * A * Arg1 / Ent + Argl9 * dEntdVds);
      double cdbb = b->cgate * (0.5 * dVthdVbs + d->vds * dAdVbs * 
			     (0.75 - 0.25 * Argl5 ) + Argl9 * dEntdVbs);
      cdsb = -(cdgb + cddb + cdbb);
      trace2("", Argl9, cdbb);
      trace4("tri 0/100", qd, cdgb, cddb, cdsb);
    }else{
      /*40/60 partitioning for drain/source chArges at saturation region*/
      double Vgs_VthSquare = Vgs_Vth*Vgs_Vth;
      trace2("", Vgs_Vth, Vgs_VthSquare);
      double Arg5 = Arg1*Arg1;
      double Vcom = Vgs_Vth*Vgs_Vth/6.0-1.25e-1*Arg1*Vgs_Vth+2.5e-2*Arg5;
      double Argl4 = Vcom/Ent/EntSquare;
      double Argl6;
      {if (Ent > 1.0e-8){
	Argl6 = Vcom / EntSquare;
      }else{   
	untested();
	Argl6 = 4.0 / 1.5e1;
      }}
      qd = -b->cgate * (0.5 * (Vgs_Vth-Arg1) + Arg1 * Argl6);
      cdgb = -b->cgate*(0.5+Arg1*(4.0*Vgs_Vth-1.5*Arg1)/Argl1-2.0*Arg1*Argl4);
      cddb = b->cgate*(0.5*A+2.0*Arg1*dEntdVds*Argl4-A*(2.0*Vgs_VthSquare
					-3.0*Arg1*Vgs_Vth+0.9*Arg5)/Argl1);
      double cdbb = b->cgate*(0.5*dVthdVbs+0.5*d->vds*dAdVbs+2.0*Arg1*dEntdVbs
	*Argl4-d->vds*(2.0*Vgs_VthSquare*dAdVbs-4.0*A*Vgs_Vth*dVthdVbs-3.0
		*Arg1*Vgs_Vth*dAdVbs+1.5*A*Arg1*dVthdVbs+0.9*Arg5*dAdVbs)
			   /Argl1);
      cdsb = -(cdgb + cddb + cdbb);
      trace4("", Vcom, Argl4, Argl6, cdbb);
      trace4("lin 40/60", qd, cdgb, cddb, cdsb);
    }}
  }else{				/* saturation region */
    assert(d->vds >= VdsPinchoff);
    double Args1 = 1.0 / (3.0 * A);
    trace2("sat", b->cgate, Args1);

    qg = b->cgate * (d->vgs - b->vfb - b->phi - Vgs_Vth * Args1);
    cggb = b->cgate * (1.0 - Args1);
    cgdb = 0.0;
    double cgbb = b->cgate * Args1 * (dVthdVbs + Vgs_Vth * dAdVbs / A);
    cgsb = -(cggb + cgdb + cgbb);
    trace4("", qg, cggb, cgdb, cgsb);

    qb = b->cgate * (b->vfb + b->phi - Vth0 + (1.0 - A) * Vgs_Vth * Args1);
    cbgb = b->cgate * (Args1 - 1.0 / 3.0);
    cbdb = 0.0;
    double cbbb = -b->cgate * ((2.0 / 3.0 + Args1) * dVthdVbs
			    + Vgs_Vth * Args1 * dAdVbs / A);
    cbsb = -(cbgb + cbdb + cbbb);
    trace4("", qb, cbgb, cbdb, cbsb);

    {if (m->xpart >= 1){
      /*0/100 partitioning for drain/source chArges at saturation region*/
      qd = 0.0;
      cdgb = 0.0;
      cddb = 0.0;
      cdsb = 0.0;
      trace4("sat 0/100", qd, cdgb, cddb, cdsb);
    }else{
      /*40/60 partitioning for drain/source chArges at saturation region*/
      const double co4v15 = 4./15.;
      qd   = -co4v15 * b->cgate * Vgs_Vth;
      cdgb = -co4v15 * b->cgate;
      cddb =  0.0;
      double cdbb = co4v15 * b->cgate * dVthdVbs;
      cdsb = -(cdgb + cddb + cdbb);
      trace4("sat 40/60", qd, cdgb, cddb, cdsb);
    }}
  }}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
