/*$Id: d_mos4.h,v 16.14 1999/12/19 22:52:34 al Exp $ -*- C++ -*-
 * MOS BSIM1 model.
 * Derived from Berkeley Spice3f4
 */
#ifndef D_MOS4_H
#define D_MOS4_H
#include "u_sdp.h"
#include "d_mos.h"
/*--------------------------------------------------------------------------*/
class SDP_MOS4 : public SDP_MOS_BASE {
public:
  explicit SDP_MOS4(const MOS_COMMON*);
public:
  //double phi; in base
  double vfb;      /* flat band voltage at given L and W    */
  double k1;       /* bulk effect coefficient 1             */
  double k2;       /* bulk effect coefficient 2             */
  double eta;      /* drain induced barrier lowering        */
  double etaB;     /* Vbs dependence of Eta                 */
  double etaD;     /* Vds dependence of Eta                 */
  
  double betaZero; /* Beta at vds = 0 and vgs = Vth         */
  double betaZeroB;/* Vbs dependence of BetaZero	    */
  double betaVdd;  /* Beta at vds=Vdd and vgs=Vth           */
  double betaVddB; /* Vbs dependence of BVdd                */
  double betaVddD; /* Vds dependence of BVdd                */
  
  double ugs;      /* Mobility degradation due to gate field*/
  double ugsB;     /* Vbs dependence of Ugs                 */
  double uds;      /* Drift Velocity Saturation due to Vds  */
  double udsB;     /* Vbs dependence of Uds                 */
  double udsD;     /* Vds dependence of Uds                 */
  double n0;	   /* slope of subthreshold current with Vgs*/
  double nB;	   /* Vbs dependence of Subthreshold Slope  */
  double nD;	   /* Vds dependence of Subthreshold Slope  */

  double vt0;
};
/*--------------------------------------------------------------------------*/
class MODEL_MOS4 : public MODEL_MOS_BASE {
  friend class SDP_MOS4;
private:
  explicit MODEL_MOS4(const MODEL_MOS4& p):MODEL_MOS_BASE(p){unreachable();}
public:
  explicit MODEL_MOS4();
private: // override virtual
  void	   parse_params(CS&);
  void	   post_parse();
  SDP_MOS_BASE* new_sdp(const MOS_COMMON* c)const {return new SDP_MOS4(c);}
  void	   print(OMSTREAM&,int)const;
  void	   tr_eval(COMPONENT*)const;
  void	   do_charge(COMPONENT*)const;
private:		/* strictly internal */
  enum {LEVEL=4};
private:		/* input parameters */
  SDP phi;		// Strong inversion surface potential
  SDP vfb;		// Flat band voltage
  SDP k1;		// Bulk effect coefficient 1
  SDP k2;		// Bulk effect coefficient 2

  SDP eta;		// VDS dependence of threshold voltage
  SDP etaB;		// VBS dependence of eta (x2e)
  SDP etaD;		// VDS dependence of eta (x3e)
  
  double mobZero;	// Zero field mobility at VDS=0 VGS=VTH (muz)
  SDP mobZeroB;		// VBS dependence of muz (x2mz)

  SDP mobVdd;		// Mobility @ VDS=VDD VGS=VTH, chan len modulation(mus)
  SDP mobVddB;		// VBS dependence of mus (x2ms)
  SDP mobVddD;		// VDS dependence of mus (x3ms)

  SDP ugs;		// VGS dependence of mobility (u0)
  SDP ugsB;		// VBS dependence of u0 (x2u0)

  SDP uds;		// VDS dependence of mobility, velocity saturation (u1)
  SDP udsB;		// VBS dependence of u1 (x2u1)
  SDP udsD;		// VDS dependence of u1 (x3u1)

  SDP n0;		// Subthreshold slope (n0)
  SDP nB;		// VBS dependence of subthreshold slope (nb)
  SDP nD;		// VDS dependence of subthreshold slope (nd)

  double dl;		// Channel length reduction in METERS (dl)
  double dw;		// Channel width reduction in METERS (dw)

  double tox;		// Gate oxide thickness in METERS (tox)
  double vdd;		// Supply voltage to specify mus
  double wdf;		// Default width of source drain diffusion (ignored)
  double dell;		// Length reduction of source drain diffusion (ignored)
  double temp;		// temperature (ignored)
  double xpart;		// Flag for channel charge partitioning

private:		/* calculated parameters */
  double cox;		/* unit: F/METERS**2 */
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
