/*$Id: d_mos5.h,v 16.14 1999/12/19 22:52:34 al Exp $ -*- C++ -*-
 * MOS BSIM2 model.
 * Derived from Berkeley Spice3f4
 */
#ifndef D_MOS5_H
#define D_MOS5_H
#include "u_sdp.h"
#include "d_mos.h"
/*--------------------------------------------------------------------------*/
class SDP_MOS5 : public SDP_MOS_BASE {
public:
  explicit SDP_MOS5(const MOS_COMMON*);
public:
  //double phi; in base
  double vfb;      	/* flat band voltage at given L and W	 */
  double k1;       	/* bulk effect coefficient 1             */
  double k2;       	/* bulk effect coefficient 2             */
  double eta0;      	/* drain induced barrier lowering        */
  double etaB;     	/* Vbs dependence of Eta                 */
  double beta0;    	/* Beta at Vds = 0 and Vgs = Vth         */
  double beta0B;   	/* Vbs dependence of Beta0               */
  double betas0;    	/* Beta at Vds=Vdd and Vgs=Vth           */
  double betasB;   	/* Vbs dependence of Betas               */
  double beta20;   	/* Vds dependence of Beta in tanh term   */
  double beta2B;   	/* Vbs dependence of Beta2               */
  double beta2G;   	/* Vgs dependence of Beta2               */
  double beta30;   	/* Vds dependence of Beta in linear term */
  double beta3B;   	/* Vbs dependence of Beta3               */
  double beta3G;   	/* Vgs dependence of Beta3               */
  double beta40;   	/* Vds dependence of Beta in quadra term */
  double beta4B;   	/* Vbs dependence of Beta4               */
  double beta4G;   	/* Vgs dependence of Beta4               */
  double ua0;      	/* Linear Vgs dependence of Mobility     */
  double uaB;      	/* Vbs dependence of Ua                  */
  double ub0;      	/* Quadratic Vgs dependence of Mobility  */
  double ubB;      	/* Vbs dependence of Ub                  */
  double u10;      	/* Drift Velocity Saturation due to Vds  */
  double u1B;      	/* Vbs dependence of U1                  */
  double u1D;      	/* Vds dependence of U1                  */
  double n0;       	/* Subthreshold slope at Vds=0, Vbs=0    */
  double nB;       	/* Vbs dependence of n                   */
  double nD;       	/* Vds dependence of n                   */
  double vof0;     	/* Vth offset at Vds=0, Vbs=0            */
  double vofB;     	/* Vbs dependence of Vof                 */
  double vofD;     	/* Vds dependence of Vof                 */
  double ai0;      	/* Pre-factor in hot-electron effects    */
  double aiB;      	/* Vbs dependence of Ai                  */
  double bi0;      	/* Exp-factor in hot-electron effects    */
  double biB;      	/* Vbs dependence of Bi                  */
  double vghigh;   	/* Upper bound of cubic spline function  */
  double vglow; 	/* Lower bound of cubic spline function  */
  double Phis3;
  double One_Third_CoxWL;
  double Two_Third_CoxWL;
  double Arg;
};
/*--------------------------------------------------------------------------*/
class MODEL_MOS5 : public MODEL_MOS_BASE {
  friend class SDP_MOS5;
private:
  explicit MODEL_MOS5(const MODEL_MOS5& p):MODEL_MOS_BASE(p){unreachable();}
public:
  explicit MODEL_MOS5();
private: // override virtual
  void	   parse_params(CS&);
  void	   post_parse();
  SDP_MOS_BASE* new_sdp(const MOS_COMMON* c)const {return new SDP_MOS5(c);}
  void	   print(OMSTREAM&,int)const;
  void	   tr_eval(COMPONENT*)const;
  void	   do_charge(COMPONENT*)const;
private:		/* strictly internal */
  enum {LEVEL=5};
private:		/* input parameters */
  SDP phi;
  SDP vfb;
  SDP k1;
  SDP k2;
  SDP eta0;
  SDP etaB;
  double mob0;
  SDP mob0B;
  SDP mobs0;
  SDP mobsB;
  SDP mob20;
  SDP mob2B;
  SDP mob2G;
  SDP mob30;
  SDP mob3B;
  SDP mob3G;
  SDP mob40;
  SDP mob4B;
  SDP mob4G;
  SDP ua0;
  SDP uaB;
  SDP ub0;
  SDP ubB;
  SDP u10;
  SDP u1B;
  SDP u1D;
  SDP n0;
  SDP nB;
  SDP nD;
  SDP vof0;
  SDP vofB;
  SDP vofD;
  SDP ai0;
  SDP aiB;
  SDP bi0;
  SDP biB;
  SDP vghigh;
  SDP vglow;

  double dl;
  double dw;

  double tox;		/* unit: METERS  */
  double vdd;
  double vgg;
  double vbb;
  double wdf;		// Default width of source drain diffusion (ignored)
  double dell;		// Length reduction of source drain diffusion (ignored)
  double temp;
  double xpart;		// Flag for channel charge partitioning


private:		/* calculated parameters */
  double cox;           /* unit: F/METERS**2 */
  double vdd2;
  double vgg2;
  double vbb2;
  double Vtm;
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
