/*$Id: e_storag.cc,v 16.16 2000/01/04 06:39:57 al Exp $ -*- C++ -*-
 * Base class for storage elements of a circuit
 */
#include "m_divdiff.h"
#include "e_storag.h"
/*--------------------------------------------------------------------------*/
/* table for selecting local integraton method
 * Determines which one wins in a conflict.
 * "only" wins over non-only.  local (_method_u) wins over opt.
 */
//                     OPT::method    _method_u
METHOD STORAGE::method_select[meNUM_METHODS][meNUM_METHODS] = {
  /*vv OPT vv*/
  //local>>>EULER,EULERONLY,TRAP,TRAPONLY,GEAR2,GEAR2ONLY,TRAPGEAR,TRAPEULER
  /*meUNKNOWN*/
  {mTRAPGEAR,mEULER,mEULER,mTRAP, mTRAP,mGEAR, mGEAR,mTRAPGEAR,mTRAPEULER},
  /*meEULER*/
  {mEULER,   mEULER,mEULER,mTRAP, mTRAP,mGEAR, mGEAR,mTRAPGEAR,mTRAPEULER},
  /*meEULERONLY*/
  {mEULER,   mEULER,mEULER,mEULER,mTRAP,mEULER,mGEAR,mEULER,   mEULER},
  /*meTRAP*/
  {mTRAP,    mEULER,mEULER,mTRAP, mTRAP,mGEAR, mGEAR,mTRAPGEAR,mTRAPEULER},
  /*meTRAPONLY*/
  {mTRAP,    mTRAP, mEULER,mTRAP, mTRAP,mTRAP, mGEAR,mTRAP,    mTRAP},
  /*meGEAR*/
  {mGEAR,    mEULER,mEULER,mTRAP, mTRAP,mGEAR, mGEAR,mTRAPGEAR,mTRAPEULER},
  /*meGEAR2ONLY*/
  {mGEAR,    mGEAR, mEULER,mGEAR, mTRAP,mGEAR, mGEAR,mGEAR,    mGEAR},
  /*meTRAPGEAR*/
  {mTRAPGEAR,mEULER,mEULER,mTRAP, mTRAP,mGEAR, mGEAR,mTRAPGEAR,mTRAPEULER},
  /*meTRAPEULER*/
  {mTRAPEULER,mEULER,mEULER,mTRAP,mTRAP,mGEAR, mGEAR,mTRAPGEAR,mTRAPEULER}
};
/*--------------------------------------------------------------------------*/
void STORAGE::precalc()
{
  {if (subckt().exists()){
    subckt().precalc();
  }else{
    _method_a = method_select[OPT::method][_method_u];
    _y0.f0 = LINEAR;
    _y0.f1 = value();
    assert(_loss0 == 0.);
    assert(_loss1 == 0.);
    set_converged();
    assert(!constant()); /* because of integration */
  }}
  /* m0 and acg are frequency/time dependent and cannot be set here.
   * If this is a coupled inductor, there is a subckt, which is expanded
   * by the mutual pseudo-element.
   * Assigning the values here becomes unnecessary, but harmless.
   */
}
/*--------------------------------------------------------------------------*/
void STORAGE::begin_core()
{
  _method_a = method_select[OPT::method][_method_u];
  if (!has_tr_eval()){
    //assert(_y0.f0 == LINEAR); f0 == f1 * x
    assert(_y0.f1 == value());
    assert(_loss0 == 0.);
    assert(_loss1 == 0.);
    assert(converged());
    assert(!constant());
  }
}
/*--------------------------------------------------------------------------*/
void STORAGE::dc_advance()
{
  {if (subckt().exists()){
    subckt().dc_advance();
  }else{
    _mt1 = _m0;
    assert(SIM::time0 == 0.);
    assert(_keep_time_steps == 4);
    _time[0] = _time[1] = _time[2] = _time[3] = 0.;
  }}
}
/*--------------------------------------------------------------------------*/
void STORAGE::tr_advance()
{
  {if (subckt().exists()){
    subckt().tr_advance();
  }else{
    {if (_time[0] < SIM::time0){		// forward
      assert(_keep_time_steps == 4);
      _mt1 = _m0;
      _yt3 = _yt2;
      _yt2 = _yt1;
      _yt1 = _y0;
      for (int i=_keep_time_steps-1; i>0; --i){
	_time[i] = _time[i-1];
      }
    }else{				// else backward, don't save
      assert(_time[1] < SIM::time0);
    }}
    _time[0] = SIM::time0;
  }}
}
/*--------------------------------------------------------------------------*/
double STORAGE::review(double i0, double it1)const
{
  assert(order() <= _max_order);
  {if (_time[order()+1] <= 0. || _method_a == mEULER){
    return NEVER;
  }else{
    double c[_keep_time_steps];
    c[0] = _y0.f0;
    c[1] = _yt1.f0;
    c[2] = _yt2.f0;
    c[3] = _yt3.f0;
    divided_differences(c, order()+2, _time);
    // now c[i] is i'th derivative (up to order-1)

    {if (c[order()+1] == 0){
      return NEVER;
    }else{
      double dt = _time[0] - _time[1];
      double currenttol= OPT::abstol+OPT::reltol*std::max(fabs(i0),fabs(it1));
      double chargetol = std::max(std::max(fabs(_y0.f0),fabs(_yt1.f0)),
				  OPT::chgtol) * OPT::reltol / dt;
      double tol = OPT::trtol * std::max(currenttol,chargetol);
      double denom = factor() * fabs(c[order()+1]);
      double timestep = pow((tol / denom), 1./order());
      
      if (timestep <= SIM::dtmin){
	untested();
	error(bDANGER,"step control error:%s %g\n",
	      long_label().c_str(),timestep);
	error(bTRACE, "q0=%g i0=%g dq0=%g\n", _y0.f0, i0, c[1]);
	error(bTRACE, "it=%g qt=%g tol=%g\n", currenttol, chargetol, tol);
	timestep = SIM::dtmin;
      }
      {if (timestep < dt * OPT::trreject){
	error(bTRACE, "step rejected:" + long_label() + '\n');
	error(bTRACE, "new=%g  old=%g  rej=%g\n",
	      timestep, dt, dt * OPT::trreject);
	return _time[1] + timestep;
      }else{
	return _time[0] + timestep;
      }}
    }}
  }}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
