/*$Id: e_storag.h,v 16.16 2000/01/04 06:39:57 al Exp $ -*- C++ -*-
 * "base" class for energy storage elements (L & C)
 */
#ifndef E_STORAGE_H
#define E_STORAGE_H
#include "e_elemnt.h"
/*--------------------------------------------------------------------------*/
enum METHOD {mTRAPGEAR, mEULER, mTRAP, mGEAR, mTRAPEULER};
/*--------------------------------------------------------------------------*/
class STORAGE : public ELEMENT {
protected:
  explicit STORAGE()			:ELEMENT(),  _method_a(mTRAPGEAR)  {}
  explicit STORAGE(const STORAGE& p)	:ELEMENT(p), _method_a(p._method_a) {}
protected: // override virtual
  void	   precalc();
  void     dc_advance();
  void     tr_advance();
  bool	   tr_needs_eval()	{return true;}
  void	   tr_queue_eval()	{q_eval();}
protected:
  void	   begin_core();
  double   review(double, double)const;
  int	   order()const {const int o[] = {1, 1, 2, 1, 1}; return o[_method_a];}
  double   factor()const
  {const double f[]={1./2., 1./2., 1./12., 1./2., 1./2.}; return f[_method_a];}
protected:
  enum {_max_order = 2, _keep_time_steps=_max_order+2};
  METHOD   _method_a;	/* actual integration method (auto)	*/
  double   _time[_keep_time_steps];
//FPOLY1   _y0;		/* iteration parameters, new		*/
  FPOLY1   _yt1;	/* iteration parameters, 1 time ago	*/
  FPOLY1   _yt2;	/* iteration parameters, 2 times ago	*/
  FPOLY1   _yt3;	/* iteration parameters, 3 times ago	*/
//FPOLY1   _y1;		/* iteration parameters, 1 iter ago	*/
//FPOLY1   _y2;		/* iteration parameters, 2 iter ago	*/
//CPOLY1   _m0;		/* matrix parameters, new		*/
//CPOLY1   _m1;		/* matrix parameters, 1 fill ago	*/
  CPOLY1   _mt1;	/* matrix parameters, 1 time ago	*/

//COMPLEX  _ev;		/* ac effective value (usually real)	*/
//COMPLEX  _acg;	/* ac admittance matrix values		*/   
protected:
  static METHOD method_select[meNUM_METHODS][meNUM_METHODS];
};
// commented out declarations mean inherited
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
